package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.data.mongodb.core.query.Criteria;

import java.util.Collections;
import java.util.Set;

/**
 * 带鉴权的查询实体，主要以部门隔绝
 *
 * @param <T>
 */
@Getter
@Setter
public abstract class AuthorizedQo<T extends AuthorizedDomain> extends NameLikeQo<T> {

    // 是否已发布
    private Boolean published;

    /**
     * 获取可见的权限ids
     *
     * @return 结果
     */
    private Set<String> authorizeIds() {
        if (user instanceof AuthorizedUserDetails) {
            return ((AuthorizedUserDetails) user).getAuthorityCodes();
        }
        return Collections.singleton(Department.PUBLIC);
    }

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        // 超级管理员拥有查看所有草稿的权限
        if (user.getType() == UserType.SUPER_ADMIN) {
            return super.criteriaBuilder().with("published");
        }
        // 查询草稿，只查询自己的
        if (BooleanUtils.isFalse(published)) {
            return super.criteriaBuilder().with("published").with(() -> Criteria.where("creatorId").is(user.getId()));
        }
        // 普通查询，根据权限配置查询
        return super.criteriaBuilder()
                .with(() -> Criteria.where("$or").is(
                        CriteriaBuilder.createCriteriaList(
                                Criteria.where("authorizeId").in(authorizeIds()),
                                Criteria.where("creatorId").is(user.getId())
                                        .and("authorizeId").in(((AuthorizedUserDetails) user).getVisibleDeparts())
                        )
                ))
                .with("published");
    }
}
